import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import { 
  csv, 
  scaleLinear, 
  max, 
  format, 
  extent, 
  scaleBand 
} from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';

const width = 960;
const menuHeight = 75;
const height = 500 - menuHeight;
const margin = {
  top: 50,
  right: 30,
  bottom: 65,
  left: 150,
};
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 100;

const attributes = [
  { value: 'MeanSepalLength', label: 'Mean Sepal Length' },
  { value: 'MeanSepalWidth', label: 'Mean Sepal Width' },
  { value: 'MeanPetalLength', label: 'Mean Petal Length' },
  { value: 'MeanPetalWidth', label: 'Mean Petal Width' },
];

const getLabel = (value) => {
  for (let i = 0; i < attributes.length; i++) {
    if (attributes[i].value == value) {
      return attributes[i].label;
    }
  }
};

const App = () => {
  const data = useData();

  const initialXAttribute = 'MeanSepalLength';
  const [xAttribute, setxAttribute] = useState(
    initialXAttribute
  );
  const xValue = (d) => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);

  const initialYAttribute = 'Species';
  const yValue = d => d.Species; 
  const yAxisLabel = 'Species';

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const xScale = scaleLinear()
  	.domain([0, max(data, xValue)])
    //.domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleBand()
  	.domain(data.map(yValue))
    .range([0, innerHeight])
  .padding(.5);

  return (
    <>
      <label for="x-select">X:</label>
      <Dropdown
        options={attributes}
        id="x-select"
        selectedValue={xAttribute}
        onSelectedValueChange={setxAttribute}
      />
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left}, ${margin.top})`}
        >
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickOffset={5}
          />
          <AxisLeft
            yScale={yScale}
            innerWidth={innerWidth}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`rotate(-90) translate(${
              -innerHeight / 2
            }, ${-yAxisLabelOffset})`}
          >
            {yAxisLabel}
          </text>
          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor={'middle'}
          >
            {xAxisLabel}
          </text>
          <Marks
            data={data}
            yScale={yScale}
            xScale={xScale}
            xValue={xValue}
            yValue={yValue}
            circleRadius={30}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
